.panel-item .panel-body .label-tag {
    white-space: normal;
}

.tag-container .label {
    margin-right: 2px;
}

.label-tag a {
    color: white;
    text-decoration: none !important;
}

.label-tag {
    text-shadow: 0px 0px 2px #000;
    box-shadow: 3px 3px 5px rgba(0, 0, 0, 0.5);
}

.select2-container--default .select2-selection--multiple .select2-selection__choice .label {
    padding-bottom: 100px;
}
