from .document_file_tasks import *  # NOQA
from .document_tasks import *  # NOQA
from .document_type_tasks import *  # NOQA
from .document_version_tasks import *  # NOQA
from .trashed_document_tasks import *  # NOQA
