from mayan.apps.appearance.classes import Icon

icon_document_download_multiple = Icon(
    driver_name='fontawesome', symbol='download'
)
icon_document_download_single = Icon(
    driver_name='fontawesome', symbol='download'
)
icon_document_file_download_quick = Icon(
    driver_name='fontawesome', symbol='download'
)
