from mayan.apps.appearance.classes import Icon

# Namespace

icon_setting_namespace_detail = Icon(
    driver_name='fontawesome', symbol='sliders-h'
)
icon_setting_namespace_list = Icon(
    driver_name='fontawesome', symbol='sliders-h'
)

# Setting

icon_setting_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
